/*
** simple.c - Trivial example showing how to read and write registers
**            in the FPGA.
**
** (C) Copyright 2009 Alpha Data
*/

#if defined(_WIN32)

/* Windows */
# include <windows.h>
# include <tchar.h>

#else

/* Linux or VxWorks */
# define _T(x) x
# define _tmain main
# define _tprintf printf
# define _tscanf_s(buffer, ...) scanf(buffer, __VA_ARGS__) /* This is OK unless the format string contains %s */
# define FALSE (0)
# define TRUE (1)

#endif

#include <stdio.h>
#include <string.h>

#include <admxrc3.h>

#include <sdk_common.h>
#include <platform_endian.h>

int
_tmain(
  int argc,
  TCHAR *argv[])
{
  int ret = 0;
  ADMXRC3_STATUS status;
  unsigned int index, serial;
  ADMXRC3_HANDLE hCard = ADMXRC3_HANDLE_INVALID_VALUE;
  ADMXRC3_CARD_INFO cardInfo;
  volatile uint32_t* pFpgaSpace = NULL;
  unsigned long dataOut, dataIn;
  const TCHAR* pFilename;
  Option options[3];
  Arguments arguments;
  bool_t bUseUber;

  /* Set up expected options */
  options[0].key = _T("index");
  options[0].type = OptionUInt;
  options[0].def.uintVal = 0;
  options[0].meaning = _T("index of card to use");
  options[1].key = _T("sn");
  options[1].type = OptionUInt;
  options[1].def.uintVal = 0;
  options[1].meaning = _T("serial number of card to use");
  options[2].key = _T("uber");
  options[2].type = OptionBoolean;
  options[2].def.booleanVal = FALSE;
  options[2].meaning = _T("use uber design instead of simple");
  arguments.nOption = 3;
  arguments.option = options;
  arguments.nParamType = 0;
  arguments.minNParam = 0;

  if (sampleParseCommandLine(_T("simple"), argc, argv, &arguments) < 0) {
    ret = -1;
    goto done;
  }
  index = options[0].value.uintVal;
  serial = options[1].value.uintVal;
  bUseUber = options[2].value.booleanVal;

  status = sampleOpenCard(index, options[0].specified, serial, options[1].specified, FALSE, &hCard);
  if (status != ADMXRC3_SUCCESS) {
    _tprintf(_T("*** Failed to open card with index %lu: %s\n"), (unsigned long)index, ADMXRC3_GetStatusString(status, TRUE));
    ret = -1;
    goto done;
  }

  status = ADMXRC3_GetCardInfo(hCard, &cardInfo);
  if (status != ADMXRC3_SUCCESS) {
    _tprintf(_T("*** Failed to get card info: %s\n"), ADMXRC3_GetStatusString(status, TRUE));
    ret = -1;
    goto done;
  }

  /* Get the address of FPGA space */
  status = ADMXRC3_MapWindow(hCard, 0, 0x0, 0x10, (void*)&pFpgaSpace);
  if (status != ADMXRC3_SUCCESS) {
    _tprintf(_T("*** Failed to map FPGA space: %s\n"), ADMXRC3_GetStatusString(status, TRUE));
    ret = -1;
    goto done;
  }

  if (cardInfo.Model < 0x100) {
    /* Set local bus clock to nominal frequency */
    status = ADMXRC3_SetClockFrequency(hCard, ADMXRC3_CLOCK_LCLK, 0, 33.0e6, NULL);
    if (status != ADMXRC3_SUCCESS) {
      _tprintf(_T("*** Failed to set LCLK to %.1lf MHz: %s\n"), 33.0, ADMXRC3_GetStatusString(status, TRUE));
      ret = -1;
      goto done;
    }
  }

  if (bUseUber) {
    pFilename = sampleGetFpgaFilename(hCard, 0, _T("uber"), _T("uber"), &arguments);
  } else {
    pFilename = sampleGetFpgaFilename(hCard, 0, _T("simple"), _T("simple"), &arguments);
  }
  if (NULL == pFilename) {
    _tprintf(_T("*** Failed to determine bitstream filename.\n"));
    ret = -1;
    goto done;
  }

  status = ADMXRC3_ConfigureFromFile(hCard, 0, 0, pFilename);
  if (status != ADMXRC3_SUCCESS) {
    _tprintf(_T("*** Failed to configure target FPGA with '%s': %s\n"), pFilename, ADMXRC3_GetStatusString(status, TRUE));
    ret = -1;
    goto done;
  }

  _tprintf(_T("====================\n"));
  _tprintf(_T("Enter values for I/O\n"));
  _tprintf(_T(" (use 55AA to exit)\n"));
  _tprintf(_T("====================\n"));

  do {
    _tscanf_s(_T("%lX"), &dataOut);
    pFpgaSpace[0] = cpu_to_le32((uint32_t)dataOut);
    dataIn = (unsigned long)pFpgaSpace[0];
    dataIn = le32_to_cpu(dataIn);
    _tprintf(_T("OUT = 0x%08lx, IN = 0x%08lx\n"), (unsigned long)dataOut, (unsigned long)dataIn);
  } while (0x55AAU != dataOut);

done:
  if (NULL != pFpgaSpace) {
    status = ADMXRC3_UnmapWindow(hCard, (void*)pFpgaSpace);
    if (status != ADMXRC3_SUCCESS) {
      _tprintf(_T("+++ Failed to unmap FPGA space: %s\n"), ADMXRC3_GetStatusString(status, TRUE));
    }
    pFpgaSpace = NULL;
  }

  if (hCard != ADMXRC3_HANDLE_INVALID_VALUE) {
    ADMXRC3_Close(hCard);
    hCard = ADMXRC3_HANDLE_INVALID_VALUE;
  }

  sampleCleanupCommandLine();
  return ret;
}
