--==============================================================================
--
--     File Name: adb3_ocp_mux.vhd
--       Version: 0.1
--          Date: 10 Jun 2010
--         Model: synth
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: OCP mux block
--
--   Description: Block giving multiple OCP Masters control over single OCP slave
--
--   Limitations: Does not allow multi-threading of commands
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 10 Jun 2010: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;

library Work;
use work.adb3_ocp.all;

entity adb3_ocp_mux is
  generic (
    mux_inputs : natural := 2);
  port(
    ocp_rst       : in    std_logic;
    ocp_clk       : in    std_logic;
    -- Upstream OCP interfaces
    ocp_m2s       : in    adb3_ocp_m2sT_array(0 to mux_inputs-1);
    ocp_s2m       : out   adb3_ocp_s2mT_array(0 to mux_inputs-1);
    -- Downstream OCP interface
    ocp_m2s_memif : out   adb3_ocp_m2sT;
    ocp_s2m_memif : in    adb3_ocp_s2mT;
    -- Status
    status_w      : out   std_logic_vector(mux_inputs-1 downto 0);
    status_r      : out   std_logic_vector(mux_inputs-1 downto 0));
end;

architecture mixed of adb3_ocp_mux is

  subtype burst_cnt_s  is std_logic_vector(ADB3_OCP_BURST_WIDTH-1 downto 0);
  subtype port_vec_s   is std_logic_vector(mux_inputs-1 downto 0);

  signal act_port         : port_vec_s;
  signal busy             : std_logic;
  signal writing          : std_logic;
  signal reading          : std_logic;
  signal ocp_m2s_memif_i  : adb3_ocp_m2sT;

  signal data_last        : std_logic;
  signal resp_last        : std_logic;
  signal wburst           : burst_cnt_s;
  signal rburst           : burst_cnt_s;
  signal wburst_completed : std_logic;
  signal rburst_completed : std_logic;

begin

  mux_m2s_l : for n in 0 to mux_inputs-1 generate
    -- Connect upstream OCP ports to downstream OCP port using act_port
    ocp_m2s_memif_i <= ocp_m2s(n) when act_port(n) = '1' else ADB3_OCP_Z_M2S;

    -- Connect downstream OCP port to upstream OCP ports using act_port
    ocp_s2m(n) <= ocp_s2m_memif when act_port(n) = '1' else ADB3_OCP_DEFAULT_S2M;

    -- Produce write/read status for all downstream ports
    status_w(n) <= writing and act_port(n);
    status_r(n) <= reading and act_port(n);
  end generate;

  ocp_m2s_memif <= ocp_m2s_memif_i;


  -- Look for next highest active port by testing in reverse order from
  -- n-1 downto 0, and then from (mux_inputs-1) downto (n+1)
  process (ocp_rst,ocp_clk)
    variable act_port_var : port_vec_s;
  begin
    if (ocp_rst = '1') then
      act_port_var     := std_logic_vector(to_unsigned(1,mux_inputs));
      act_port         <= std_logic_vector(to_unsigned(1,mux_inputs));
      writing          <= '0';
      reading          <= '0';
      wburst           <= (others => '0');
      rburst           <= (others => '0');
      wburst_completed <= '0';
      rburst_completed <= '0';
    elsif rising_edge(ocp_clk) then
      act_port_l : for n in 0 to mux_inputs-1 loop
        if (act_port(n) = '1') then
          if (ocp_m2s(n).Cmd /= ocp_Cmd_WRITE)and(ocp_m2s(n).Cmd /= ocp_Cmd_READ) then
            if (busy = '0') then
              -- Test for activity on mux inputs (n-1) downto 0
              if (n > 0) then
                rem_port2_l : for m in n-1 downto 0 loop
                  if (ocp_m2s(m).Cmd = ocp_Cmd_WRITE)or(ocp_m2s(m).Cmd = ocp_Cmd_READ) then
                    act_port_var := std_logic_vector(to_unsigned(2**m,mux_inputs));
                  end if;
                end loop;
              end if;
              -- Test for activity on mux inputs (mux_inputs-1) to (n+1)
              if (n < mux_inputs-1) then
                rem_port1_l : for m in mux_inputs-1 downto n+1 loop
                  if (ocp_m2s(m).Cmd = ocp_Cmd_WRITE)or(ocp_m2s(m).Cmd = ocp_Cmd_READ) then
                    act_port_var := std_logic_vector(to_unsigned(2**m,mux_inputs));
                  end if;
                end loop;
              end if;
            end if;
          end if;
        end if;
      end loop;

      act_port <= act_port_var;

      -- Memory interface Write transaction
      if (ocp_m2s_memif_i.Cmd = OCP_CMD_WRITE)and(busy = '0') then
        writing <= '1';
      elsif ((data_last = '1')and(ocp_m2s_memif_i.DataValid = '1')and(ocp_s2m_memif.DataAccept = '1'))or(wburst_completed = '1') then
        writing <= '0';
      end if;

      -- Memoty interface Write transaction
      -- Generate burst counter wburst and wburst_completed
      wburst_completed <= '0';
      if (ocp_m2s_memif_i.Cmd = OCP_CMD_WRITE)and(ocp_s2m_memif.CmdAccept = '1') then
        if (ocp_m2s_memif_i.DataValid = '1')and(ocp_s2m_memif.DataAccept = '1') then
          wburst <= ocp_m2s_memif_i.BurstLength-1;
          if (ocp_m2s_memif_i.BurstLength = std_logic_vector(to_unsigned(1,ADB3_OCP_BURST_WIDTH))) then
            wburst_completed <= '1';
          end if;
        else
          wburst <= ocp_m2s_memif_i.BurstLength;
        end if;
      else
        if (ocp_m2s_memif_i.DataValid = '1')and(ocp_s2m_memif.DataAccept = '1') then
          wburst <= wburst-1;
        end if;
      end if;

      -- Memory interface Read transaction
      if (ocp_m2s_memif_i.Cmd = OCP_CMD_READ)and(busy = '0') then
        reading <= '1';
      elsif ((resp_last = '1')and(ocp_m2s_memif_i.RespAccept = '1')and(ocp_s2m_memif.Resp = OCP_RESP_VALID))or(rburst_completed = '1') then
        reading <= '0';
      end if;

      -- Memoty interface Read transaction
      -- Generate burst counter wburst and wburst_completed
      rburst_completed <= '0';
      if (ocp_m2s_memif_i.Cmd = OCP_CMD_READ)and(ocp_s2m_memif.CmdAccept = '1') then
        if (ocp_m2s_memif_i.RespAccept = '1')and(ocp_s2m_memif.Resp = OCP_RESP_VALID)then
          rburst <= ocp_m2s_memif_i.BurstLength-1;
          if (ocp_m2s_memif_i.BurstLength = std_logic_vector(to_unsigned(1,ADB3_OCP_BURST_WIDTH))) then
            rburst_completed <= '1';
          end if;
        else
          rburst <= ocp_m2s_memif_i.BurstLength;
        end if;
      else
        if (ocp_m2s_memif_i.RespAccept = '1')and(ocp_s2m_memif.Resp = OCP_RESP_VALID) then
          rburst <= rburst-1;
        end if;
      end if;

    end if;
  end process;

  busy <= writing or reading;

  data_last <= '1' when wburst = std_logic_vector(to_unsigned(1,ADB3_OCP_BURST_WIDTH)) else '0';
  resp_last <= '1' when rburst = std_logic_vector(to_unsigned(1,ADB3_OCP_BURST_WIDTH)) else '0';


  -- Report if more that one bit of act_port is active
  act_port_check_p : process(act_port)
    variable act_port_cnt : natural;
  begin
    act_port_cnt := 0;
    gen_act_port_check_l: for i in 0 to mux_inputs-1 loop
      if (act_port(i) = '1') then
        act_port_cnt := act_port_cnt + 1;
      end if;
    end loop;
    assert not(act_port_cnt > 1)
      report "(adb3_ocp_mux): Multiple OCP ports decoded as active in act_port"
      severity failure;
  end process;

end mixed;
