--
-- adb3_ocp_pkg.vhd
-- Package For ADB3 OCP Type
--
-- (C) 2009 Alpha Data Ltd.
--


library IEEE;
use IEEE.std_logic_1164.all;

package adb3_ocp is

  constant ADB3_OCP_DATA_WIDTH  : integer := 128;
  constant ADB3_OCP_ADDR_WIDTH  : integer := 64;
  constant ADB3_OCP_BURST_WIDTH : integer := 12;
  constant ADB3_OCP_TAG_WIDTH   : integer := 8;
  constant ADB3_OCP_CMD_WIDTH   : integer := 3;
  constant ADB3_OCP_RESP_WIDTH  : integer := 2;

  subtype ocp_CmdT is std_logic_vector(ADB3_OCP_CMD_WIDTH-1 downto 0);
  constant OCP_CMD_IDLE         : ocp_CmdT := "000";
  constant OCP_CMD_WRITE        : ocp_CmdT := "001";
  constant OCP_CMD_READ         : ocp_CmdT := "010";
  constant OCP_CMD_WRITENONPOST : ocp_CmdT := "101";

  subtype ocp_RespT is std_logic_vector(ADB3_OCP_RESP_WIDTH-1 downto 0);
  constant OCP_RESP_NONE   : ocp_RespT := "00";
  constant OCP_RESP_VALID  : ocp_RespT := "01";
  constant OCP_RESP_FAILED : ocp_RespT := "10";
  constant OCP_RESP_ERROR  : ocp_RespT := "11";

  function to_std_logic(bool : boolean) return std_logic;


  -- Burst Capable bus for high performance data flows
  -- Block Data Flow from Master to Slave
  type adb3_ocp_m2sT is record
    -- Request group
    Cmd         : ocp_CmdT;   -- Command
    Addr        : std_logic_vector(ADB3_OCP_ADDR_WIDTH-1 downto 0);
    BurstLength : std_logic_vector(ADB3_OCP_BURST_WIDTH-1 downto 0);
    -- DataHandshake group
    Data        : std_logic_vector(ADB3_OCP_DATA_WIDTH-1 downto 0);
    DataByteEn  : std_logic_vector(ADB3_OCP_DATA_WIDTH/8-1 downto 0);
    DataValid   : std_logic;  -- Qualifies Data and the other signals in bundle
    -- Response group
    RespAccept  : std_logic;  -- Master indication read data was accepted
    -- Tag group
    Tag         : std_logic_vector(ADB3_OCP_TAG_WIDTH-1 downto 0);
  end record adb3_ocp_m2sT;

  -- Block Data Flow from Slave to Master
  type adb3_ocp_s2mT is record
    -- Request group
    CmdAccept  : std_logic;   -- Slave indication command/request accepted
    -- DataHandshake group
    DataAccept : std_logic;   -- Slave indication write data accepted
    -- Response group
    Data       : std_logic_vector(ADB3_OCP_DATA_WIDTH-1 downto 0);
    Resp       : ocp_RespT;   -- Slave read response
    -- Tag group
    Tag        : std_logic_vector(ADB3_OCP_TAG_WIDTH-1 downto 0);
  end record adb3_ocp_s2mT;

  type adb3_ocp_s2mT_array is array (natural range <>) of adb3_ocp_s2mT;
  type adb3_ocp_m2sT_array is array (natural range <>) of adb3_ocp_m2sT;


  -- Define type defaults
  constant ADB3_OCP_DEFAULT_M2S : adb3_ocp_m2sT :=
    (Cmd         => OCP_CMD_IDLE,
     Addr        => (others => '0'),
     BurstLength => (others => '0'),
     Data        => (others => '0'),
     DataByteEn  => (others => '0'),
     DataValid   => '0',
     RespAccept  => '0',
     Tag         => (others => '0'));

  constant ADB3_OCP_DEFAULT_S2M : adb3_ocp_s2mT :=
    (CmdAccept  => '0',
     DataAccept => '0',
     Data       => (others => '0'),
     Resp       => OCP_RESP_NONE,
     Tag        => (others => '0'));

  -- Define type tri-states
  constant ADB3_OCP_Z_M2S : adb3_ocp_m2sT :=
    (Cmd         => (others => 'Z'),
     Addr        => (others => 'Z'),
     BurstLength => (others => 'Z'),
     Data        => (others => 'Z'),
     DataByteEn  => (others => 'Z'),
     DataValid   => 'Z',
     RespAccept  => 'Z',
     Tag         => (others => 'Z'));

  constant ADB3_OCP_Z_S2M : adb3_ocp_s2mT :=
    (CmdAccept  => 'Z',
     DataAccept => 'Z',
     Data       => (others => 'Z'),
     Resp       => (others => 'Z'),
     Tag        => (others => 'Z'));

end package adb3_ocp;

package body adb3_ocp is

  function to_std_logic (bool : boolean) return std_logic is
  begin
    if (bool) then
      return '1';
    else
      return '0';
    end if;
  end function to_std_logic;

end package body adb3_ocp;

