--==============================================================================
--
--     File Name: adb3_target_inc_sim_ocp_6t1_pkg.vhd
--       Version: 0.1
--          Date: 01 Jun 2010
--         Model: Simulation (NOT SYNTHESISABLE)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_inc_pkg
--
--   Description: Package used in target FPGAs during simulation using OCP
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 01 Jun 10: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;

library adcommon;
use adcommon.clock_speed_pkg.all;


package adb3_target_inc_pkg is

  -- Select board type
  constant BOARD_TYPE : board_type_t := ADM_XRC_6T1;

  -- Select target use
  -- SIM_OCP:  Generate simulation code. Direct OCP to OCP connections between bridge (TB) and target (ie bypass MPTL)
  -- SIM_MPTL: Generate simulation code. Full OCP to MPTL to OCP connections between bridge (TB) and target
  -- SYN_NGC:  Generate synthesis code.  Instantiate .ngc core for OCP-MPTL interface to bridge
  constant TARGET_USE : target_use_t := SIM_OCP;

  -- Define OCP Channels
  constant DS_CHANNELS  : natural range 1 to MAX_DS_CHANNELS  := 1;
  constant DMA_CHANNELS : natural range 1 to MAX_DMA_CHANNELS := 4;
  constant DM_CHANNELS  : natural range 1 to MAX_DM_CHANNELS  := 1;

  -- Define ref_clk frequency of 200M MHz for clock speed measurement
  constant REF_CLK_FREQ_HZ : natural := 200000000;

  -- Select sample period for clock speed measurement = small (freq in MHz) for simulation
  constant FREQ_MEAS_TYPE : freq_meas_type_t := FREQ_IN_MHZ;


  -- Non-MGT clock inputs
  type clks_non_mgt_t is record
    ref_clk   : std_logic_dbl_t;
    lclk_s    : std_logic;
    lclk_d    : std_logic_dbl_t;
    xrm_clkin : std_logic_dbl_t;
  end record;

  -- MGT clock inputs
  type clks_mgt_t is record
    mgt112_clk0 : std_logic_dbl_t;
    mgt113_clk0 : std_logic_dbl_t;
    mgt113_clk1 : std_logic_dbl_t;
    mgt114_clk0 : std_logic_dbl_t;
    mgt115_clk0 : std_logic_dbl_t;
    mgt116_clk0 : std_logic_dbl_t;
    mgt117_clk0 : std_logic_dbl_t;
  end record;


  -- MPTL data interface
  type mptl_pins_t is record
    -- MPTL simulation/synthesis using MPTL parallel interface
    d                  : std_logic_vector(31 downto 0);
    v                  : std_logic;
    ck                 : std_logic_vector(1 downto 0);
    -- MPTL simulation/synthesis using MPTL serial interface
    p                  : std_logic_vector(MAX_MPTL_SER_WIDTH-1 downto 0);
    n                  : std_logic_vector(MAX_MPTL_SER_WIDTH-1 downto 0);
    -- MPTL simulation using OCP bypass (t2b)
    target_ocp_clk     : std_logic;
    direct_slave_s2m   : adb3_ocp_s2mT;
    dma_channels_s2m   : adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    direct_masters_m2s : adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
    gpio_t2b           : std_logic_vector(15 downto 0);
    -- MPTL simulation using OCP bypass (b2t)
    direct_slave_m2s   : adb3_ocp_m2sT;
    dma_channels_m2s   : adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    direct_masters_s2m : adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
    gpio_b2t           : std_logic_vector(15 downto 0);
  end record;

  -- MPTL clock interface
  type mptl_clk_t is record
    -- Double ended
    p   : std_logic;
    n   : std_logic;
    -- Single ended
    clk : std_logic;
  end record;

  -- mptl_dma_abort_t width = number of DMA channels
  subtype mptl_dma_abort_t is std_logic_vector(3 downto 0);

  -- XRM GPIO interface
  type xrm_gpio_t is record
    xrm_da_p    : std_logic_vector(15 downto 0);
    xrm_da_n    : std_logic_vector(15 downto 0);
    xrm_db_p    : std_logic_vector(15 downto 0);
    xrm_db_n    : std_logic_vector(15 downto 0);
    xrm_dc_p    : std_logic_vector(15 downto 0);
    xrm_dc_n    : std_logic_vector(15 downto 0);
    xrm_dd_p    : std_logic_vector(15 downto 0);
    xrm_dd_n    : std_logic_vector(15 downto 0);
    xrm_da_cc_p : std_logic;
    xrm_da_cc_n : std_logic;
    xrm_db_cc_p : std_logic;
    xrm_db_cc_n : std_logic;
    xrm_dc_cc_p : std_logic;
    xrm_dc_cc_n : std_logic;
    xrm_dd_cc_p : std_logic;
    xrm_dd_cc_n : std_logic;
    xrm_sa      : std_logic_vector(1 downto 0);
    xrm_sb      : std_logic_vector(1 downto 0);
    xrm_sc      : std_logic_vector(1 downto 0);
    xrm_sd      : std_logic_vector(3 downto 0);
  end record;


  -- XRM MGT interface
  constant XRM_MGT_WIDTH : natural range 1 to MAX_XRM_MGT_WIDTH := 8;

  type xrm_mgt_tx_t is record
    mgt_p : std_logic_vector(XRM_MGT_WIDTH-1 downto 0);
    mgt_n : std_logic_vector(XRM_MGT_WIDTH-1 downto 0);
  end record;

  type xrm_mgt_rx_t is record
    mgt_p : std_logic_vector(XRM_MGT_WIDTH-1 downto 0);
    mgt_n : std_logic_vector(XRM_MGT_WIDTH-1 downto 0);
  end record;


  -- PN4 GPIO interface
  constant PN4_GPIO_WIDTH : natural range 1 to MAX_PN4_GPIO_WIDTH := 32;

  type pn4_gpio_t is record
    gpio_p : std_logic_vector(PN4_GPIO_WIDTH downto 1);
    gpio_n : std_logic_vector(PN4_GPIO_WIDTH downto 1);
  end record;

  constant PN4_GPIO_Z : pn4_gpio_t :=
    (gpio_p => (others => 'Z'),
     gpio_n => (others => 'Z'));


  -- PN6 GPIO interface
  constant PN6_GPIO_WIDTH : natural range 1 to MAX_PN6_GPIO_WIDTH := 38;

  type pn6_gpio_t is record
    gpio : std_logic_vector(PN6_GPIO_WIDTH downto 1);
  end record;

  constant PN6_GPIO_Z : pn6_gpio_t :=
    (gpio => (others => 'Z'));


  -- PN6 MGT interface
  constant PN6_MGT_WIDTH : natural range 1 to MAX_PN6_MGT_WIDTH := 8;

  type pn6_mgt_tx_t is record
    mgt_p : std_logic_vector(PN6_MGT_WIDTH-1 downto 0);
    mgt_n : std_logic_vector(PN6_MGT_WIDTH-1 downto 0);
  end record;

  type pn6_mgt_rx_t is record
    mgt_p : std_logic_vector(PN6_MGT_WIDTH-1 downto 0);
    mgt_n : std_logic_vector(PN6_MGT_WIDTH-1 downto 0);
  end record;

  -- Single ended clock to mptl_clk_t defined above
  function sgl_to_mptl_clk_t(
    constant clk : in std_logic)
  return mptl_clk_t;

  -- Double ended clock to mptl_clk_t defined above
  function dbl_to_mptl_clk_t(
    constant clk : in std_logic_dbl_t)
  return mptl_clk_t;

end;


package body adb3_target_inc_pkg is

  function sgl_to_mptl_clk_t(
    constant clk : in std_logic)
  return mptl_clk_t is
    variable mptl_clk : mptl_clk_t;
  begin
    mptl_clk.clk := clk;

    return mptl_clk;
  end;

  function dbl_to_mptl_clk_t(
    constant clk : in std_logic_dbl_t)
  return mptl_clk_t is
    variable mptl_clk : mptl_clk_t;
  begin
    mptl_clk.p := clk.p;
    mptl_clk.n := clk.n;

    return mptl_clk;
  end;

end;