--==============================================================================
--
--     File Name: blk_direct_slave.vhd
--       Version: 0.1
--          Date: 15 Mar 2010
--         Model: synth
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Direct Slave functional block
--                Functions:
--                  Direct Slave clock domain transfer
--                  Direct Slave address space split
--                  simple test block
--                  clock frequency measurement block
--                  io test block
--                  interrupt test block
--                  info register block
--                  BRAM interface clock domain transfer
--
--   Description: Adds OCP Direct Slave interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 15 Mar 2010: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;
use work.uber_pkg.all;

entity blk_direct_slave is
port(
  -- Clock interface
  pll_ref_rst         : in    std_logic;
  pll_ref_clk         : in    std_logic;
  pll_usr_rst         : in    std_logic;
  pll_usr_clk         : in    std_logic;
  -- Direct Slave Upstream OCP Interface
  direct_slave_m2s    : in    adb3_ocp_m2sT;
  direct_slave_s2m    : out   adb3_ocp_s2mT;
  -- Clock frequency measurement register interface
  clk_vec             : in    clk_vec_t;
  -- IO signal test register interface
  xrm_gpio            : inout xrm_gpio_t;
  pn4_gpio            : inout pn4_gpio_t;
  pn6_gpio            : inout pn6_gpio_t;
  -- Interrupt test register interface
  interrupt           : out   std_logic;
  -- BRAM Downstream OCP interface
  ds_bram_m2s         : out   adb3_ocp_m2sT;
  ds_bram_s2m         : in    adb3_ocp_s2mT);
end;


architecture synth of blk_direct_slave is

  constant SIMPLE_ADDR_WIDTH   : natural := ds_vec_width(SIMPLE_ADDR_RANGE.addr_mask);
  constant CLK_READ_ADDR_WIDTH : natural := ds_vec_width(CLK_READ_ADDR_RANGE.addr_mask);
  constant IO_ADDR_WIDTH       : natural := ds_vec_width(IO_ADDR_RANGE.addr_mask);
  constant INT_ADDR_WIDTH      : natural := ds_vec_width(INT_ADDR_RANGE.addr_mask);
  constant INFO_ADDR_WIDTH     : natural := ds_vec_width(INFO_ADDR_RANGE.addr_mask);

  -- Direct slave OCP interface in pll_ref_clk domain
  signal ds_s2m        : adb3_ocp_s2mT;
  signal ds_m2s        : adb3_ocp_m2sT;

  -- Direct slave OCP interface in pll_usr_clk domain
  signal ds_reg_s2m    : adb3_ocp_s2mT;
  signal ds_reg_m2s    : adb3_ocp_m2sT;

  -- OCP split signals
  signal ds_slaves_m2s : adb3_ocp_m2sT_array(0 to NUMBER_OF_SLAVES-1);
  signal ds_slaves_s2m : adb3_ocp_s2mT_array(0 to NUMBER_OF_SLAVES-1);
  signal ds_w_oor      : std_logic;
  signal ds_r_oor      : std_logic;

begin

  ds_m2s <= direct_slave_m2s;
  direct_slave_s2m <= ds_s2m;

  -- Transfer direct slave OCP interface to pll_usr_clk domain
  ds_cross_clk_dom_i : adb3_ocp_cross_clk_dom
  port map(
    -- Upstream OCP interface
    master_rst    => pll_ref_rst,  -- in    std_logic
    master_clk    => pll_ref_clk,  -- in    std_logic
    master_m2s    => ds_m2s,       -- in    adb3_ocp_m2sT
    master_s2m    => ds_s2m,       -- out   adb3_ocp_s2mT
    master_bl_err => open,         -- out   std_logic
    -- Downstream OCP interface
    slave_rst     => pll_usr_rst,  -- in    std_logic
    slave_clk     => pll_usr_clk,  -- in    std_logic
    slave_m2s     => ds_reg_m2s,   -- out   adb3_ocp_m2sT
    slave_s2m     => ds_reg_s2m,   -- in    adb3_ocp_s2mT
    slave_bl_err  => open);        -- out   std_logic)


--  ----------------------------------------
--  -- Chipscope OCP interface connection --
--  ----------------------------------------
--
--  blk_chipscope_i : blk_chipscope
--  port map(
--    -- OCP Channel 0 Interface
--    ocp_ch0_clk   => pll_ref_clk,          -- in    std_logic
--    ocp_ch0_m2s   => ds_m2s,               -- in    adb3_ocp_m2sT
--    ocp_ch0_s2m   => ds_s2m,               -- in    adb3_ocp_s2mT
--    ocp_ch0_trig  => ocp_ch0_trig,         -- in    std_logic
--    -- OCP Channel 1 Interface
--    ocp_ch1_clk   => pll_usr_clk,          -- in    std_logic
--    ocp_ch1_m2s   => ds_reg_m2s,           -- in    adb3_ocp_m2sT
--    ocp_ch1_s2m   => ds_reg_s2m,           -- in    adb3_ocp_s2mT
--    ocp_ch1_trig  => ocp_ch1_trig,         -- in    std_logic
--    -- OCP Channel 2 Interface
--    ocp_ch2_clk   => pll_usr_clk,          -- in    std_logic
--    ocp_ch2_m2s   => ds_slaves_m2s(5),     -- in    adb3_ocp_m2sT
--    ocp_ch2_s2m   => ds_slaves_s2m(5),     -- in    adb3_ocp_s2mT
--    ocp_ch2_trig  => ocp_ch2_trig);        -- in    std_logic)


  -- Split direct slave OCP interface according to function
  adb3_ocp_reg_split_i : adb3_ocp_reg_split
  generic map(
    addr_range_table => ADDR_RANGE_TABLE)  -- addr_range_table_t
  port map(
    -- OCP Interface
    ocp_rst     => pll_usr_rst,      -- in    std_logic
    ocp_clk     => pll_usr_clk,      -- in    std_logic
    master_m2s  => ds_reg_m2s,       -- in    adb3_ocp_m2sT
    master_s2m  => ds_reg_s2m,       -- out   adb3_ocp_s2mT
    slaves_m2s  => ds_slaves_m2s,    -- out   adb3_ocp_m2sT_array(0 to addr_range_table'length-1)
    slaves_s2m  => ds_slaves_s2m,    -- in    adb3_ocp_s2mT_array(0 to addr_range_table'length-1)
    -- Status interface
    w_addr_oor  => ds_w_oor,         -- out   std_logic)
    r_addr_oor  => ds_r_oor);        -- out   std_logic)


  -----------------------
  -- Simple test block --
  -----------------------
  blk_ds_simple_test_i : blk_ds_simple_test
  generic map(
    addr_width => SIMPLE_ADDR_WIDTH)  -- natural range 1 to DS_ADDR_WIDTH)
  port map(
    -- OCP Interface
    ocp_rst          => pll_usr_rst,       -- in    std_logic
    ocp_clk          => pll_usr_clk,       -- in    std_logic
    direct_slave_m2s => ds_slaves_m2s(0),  -- in    adb3_ocp_m2sT
    direct_slave_s2m => ds_slaves_s2m(0)); -- out   adb3_ocp_s2mT)


  ---------------------------
  -- Clock frequency block --
  ---------------------------
  blk_ds_clk_read_i : blk_ds_clk_read
  generic map(
    addr_width => CLK_READ_ADDR_WIDTH)  -- natural range 1 to DS_ADDR_WIDTH)
  port map(
    -- OCP Interface
    ocp_rst          => pll_usr_rst,      -- in    std_logic
    ocp_clk          => pll_usr_clk,      -- in    std_logic
    direct_slave_m2s => ds_slaves_m2s(1), -- in    adb3_ocp_m2sT
    direct_slave_s2m => ds_slaves_s2m(1), -- out   adb3_ocp_s2mT
    -- Clock inputs
    pll_ref_clk      => pll_ref_clk,      -- in    std_logic
    pll_usr_clk      => pll_usr_clk,      -- in    std_logic
    clk_vec          => clk_vec);         -- in    clk_vec_t


  ------------------------------
  -- Front/Rear IO test block --
  ------------------------------
  blk_ds_io_test_i : blk_ds_io_test
  generic map(
    addr_width => IO_ADDR_WIDTH)  -- natural range 1 to DS_ADDR_WIDTH)
  port map(
    -- OCP Interface
    ocp_rst          => pll_usr_rst,       -- in    std_logic
    ocp_clk          => pll_usr_clk,       -- in    std_logic
    direct_slave_m2s => ds_slaves_m2s(2),  -- in    adb3_ocp_m2sT
    direct_slave_s2m => ds_slaves_s2m(2),  -- out   adb3_ocp_s2mT
    -- IO Interface
    xrm_gpio         => xrm_gpio,          -- inout xrm_gpio_t
    pn4_gpio         => pn4_gpio,          -- inout pn4_gpio_t
    pn6_gpio         => pn6_gpio);         -- inout pn6_gpio_t)


  --------------------------
  -- Interrupt test block --
  --------------------------
  blk_ds_int_test_i : blk_ds_int_test
  generic map(
    addr_width => INT_ADDR_WIDTH)  -- natural range 1 to DS_ADDR_WIDTH)
  port map(
    -- OCP Interface
    ocp_rst          => pll_usr_rst,       -- in    std_logic
    ocp_clk          => pll_usr_clk,       -- in    std_logic
    direct_slave_m2s => ds_slaves_m2s(3),  -- in    adb3_ocp_m2sT
    direct_slave_s2m => ds_slaves_s2m(3),  -- out   adb3_ocp_s2mT
    -- Interrupt interface
    interrupt        => interrupt);        -- out   std_logic)


  ----------------
  -- Info block --
  ----------------
  blk_ds_info_i : blk_ds_info
  generic map(
    addr_width => INFO_ADDR_WIDTH)  -- natural range 1 to DS_ADDR_WIDTH)
  port map(
    -- OCP Interface
    ocp_rst          => pll_usr_rst,       -- in    std_logic
    ocp_clk          => pll_usr_clk,       -- in    std_logic
    direct_slave_m2s => ds_slaves_m2s(4),  -- in    adb3_ocp_m2sT
    direct_slave_s2m => ds_slaves_s2m(4),  -- out   adb3_ocp_s2mT
    -- Info Interface
    ds_w_oor         => ds_w_oor,          -- in    std_logic
    ds_r_oor         => ds_r_oor);         -- in    std_logic)


  --------------------------
  -- BRAM block interface --
  --------------------------
  bram_cross_clk_dom_i : adb3_ocp_cross_clk_dom
  port map(
    -- Upstream OCP interface
    master_rst    => pll_usr_rst,        -- in    std_logic
    master_clk    => pll_usr_clk,        -- in    std_logic
    master_m2s    => ds_slaves_m2s(5),   -- in    adb3_ocp_m2sT
    master_s2m    => ds_slaves_s2m(5),   -- out   adb3_ocp_s2mT
    master_bl_err => open,               -- in    std_logic
    -- Downstream OCP interface
    slave_rst     => pll_ref_rst,        -- in    std_logic
    slave_clk     => pll_ref_clk,        -- in    std_logic
    slave_m2s     => ds_bram_m2s,        -- out   adb3_ocp_m2sT
    slave_s2m     => ds_bram_s2m,        -- in    adb3_ocp_s2mT)
    slave_bl_err  => open);              -- in    std_logic

end;