--==============================================================================
--
--     File Name: blk_ds_info.vhd
--       Version: 0.1
--          Date: 11 Jun 2010
--         Model: synth
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Info
--
--   Description: Provides misc info on DS OCP
--                Connected to Direct Slave OCP interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 11 Jun 2010: Initial design
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_pkg.all;
use work.uber_pkg.all;
use work.today_pkg.all;

entity blk_ds_info is
generic(
  addr_width : natural range 1 to DS_ADDR_WIDTH);
port(
  -- OCP Interface
  ocp_rst          : in    std_logic;
  ocp_clk          : in    std_logic;
  direct_slave_m2s : in    adb3_ocp_m2sT;
  direct_slave_s2m : out   adb3_ocp_s2mT;
  -- Info Interface
  ds_w_oor         : in    std_logic;
  ds_r_oor         : in    std_logic);
end;


architecture synth of blk_ds_info is

  constant REG_WIDTH : natural := 32;
  constant WE_WIDTH  : natural := REG_WIDTH/8;

  -- Register interface signals
  signal ds_d  : std_logic_vector(REG_WIDTH-1 downto 0);
  signal ds_q  : std_logic_vector(REG_WIDTH-1 downto 0);
  signal ds_a  : std_logic_vector(addr_width-1 downto 0);
  signal ds_w  : std_logic;
  signal ds_r  : std_logic;
  signal ds_we : std_logic_vector(WE_WIDTH-1 downto 0);

  -- Address OOR counters
  signal ds_w_oor_cnt : std_logic_vector(15 downto 0);
  signal ds_r_oor_cnt : std_logic_vector(15 downto 0);

begin

  --------------------------------
  -- Direct Slave OCP interface --
  --------------------------------

  adb3_ocp_simple_bus_if_i : adb3_ocp_simple_bus_if
  generic map(
    addr_width => addr_width,   -- natural range 1 to ADB3_OCP_ADDR_WIDTH
    data_width => REG_WIDTH)    -- natural range 8 to ADB3_OCP_DATA_WIDTH
  port map(
    -- OCP Interface
    ocp_rst => ocp_rst,           -- in    std_logic
    ocp_clk => ocp_clk,           -- in    std_logic
    ocp_m2s => direct_slave_m2s,  -- in    adb3_ocp_m2sT
    ocp_s2m => direct_slave_s2m,  -- out   adb3_ocp_s2mT
    -- Simple bus interface
    d       => ds_d,        -- out   std_logic_vector(data_width-1 downto 0)
    q       => ds_q,        -- in    std_logic_vector(data_width-1 downto 0)
    a       => ds_a,        -- out   std_logic_vector(addr_width-1 downto 0)
    w       => ds_w,        -- out   std_logic
    r       => ds_r,        -- out   std_logic
    we      => ds_we);      -- out   std_logic_vector((data_width/8)-1 downto 0))


  -- Multiplex read registers
  rd_regs_p : process(ocp_rst,ocp_clk)
  begin
    if (ocp_rst = '1') then
      ds_q <= (others => '0');
    elsif rising_edge(ocp_clk) then
      if    (ds_a = INFO_DATE_REG_ADDR(addr_width-1 downto 0)) then
        ds_q <= TODAYS_DATE;
      elsif (ds_a = INFO_TIME_REG_ADDR(addr_width-1 downto 0)) then
        ds_q <= TODAYS_TIME;
      elsif (ds_a = INFO_SPLIT_REG_ADDR(addr_width-1 downto 0)) then
        ds_q <= ds_w_oor_cnt & ds_r_oor_cnt;
      elsif (ds_a = INFO_BASE_REG_ADDR(addr_width-1 downto 0)) then
        ds_q(DS_ADDR_WIDTH-1 downto 0) <= BRAM_ADDR_BASE;
      elsif (ds_a = INFO_MASK_REG_ADDR(addr_width-1 downto 0)) then
        ds_q(DS_ADDR_WIDTH-1 downto 0) <= BRAM_ADDR_MASK;
      else
        ds_q <= (others => '-');
      end if;
    end if;
  end process;

  oor_cnt_p : process(ocp_rst,ocp_clk)
  begin
    if (ocp_rst = '1') then
      ds_w_oor_cnt <= (others => '0');
      ds_r_oor_cnt <= (others => '0');
    elsif rising_edge(ocp_clk) then
      if (ds_w_oor = '1') then
        ds_w_oor_cnt <= ds_w_oor_cnt + 1;
      end if;
      if (ds_r_oor = '1') then
        ds_r_oor_cnt <= ds_r_oor_cnt + 1;
      end if;
    end if;
  end process;

end;