--==============================================================================
--
--     File Name: test_uber.vhd
--       Version: 0.1
--          Date: 26 Nov 2009
--         Model: Behavioral (NOT SYNTHESISABLE)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: ADB3 Target Design Test: uber
--
--   Description: Illustrates with ADB3:
--                  Register access using direct slave interface
--                  BRAM access using direct slave interface
--                  Multi-channel DMA
--                  Direct master interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2009 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 26 Oct 09: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.mptl.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_tb_pkg.all;
use work.uber_tb_pkg.all;

entity test_uber is
end test_uber;

architecture test of test_uber is

  -- Constant for MPTL GPIO loopback test
  constant MPTL_GPIO : std_logic_vector(15 downto 0) := X"F1D0";

  -- Test clocks
  signal clk_250 : std_logic_dbl_t := ('1','0');
  signal clk_200 : std_logic_dbl_t := ('1','0');
  signal clk_125 : std_logic_dbl_t := ('1','0');
  signal clk_100 : std_logic_dbl_t := ('1','0');
  signal clk_80  : std_logic_dbl_t := ('1','0');
  signal clk_xrm : std_logic_dbl_t := ('1','0');

  -- Test results
  signal ds_comp  : ds_comp_t;
  signal ds_pass  : ds_pass_t;
  signal dma_comp : dma_comp_t;
  signal dma_pass : dma_pass_t;

  signal test_complete : boolean := false;
  signal test_passed   : boolean := true;

  -- Bridge clocks
  signal ocp_clk  : std_logic;
  signal mptl_clk : mptl_clk_t;

  -- Bridge MPTL Sideband signals
  signal b_gpio_b2t,b_gpio_t2b : std_logic_vector(15 downto 0);
  signal b_mptl_status         : std_logic_vector(63 downto 0);
  signal b_mptl_config         : std_logic_vector(31 downto 0);

  -- Probe errors
  signal probe_ds_err  : std_logic;
  signal probe_dm_err  : std_logic;
  signal probe_dma_err : std_logic;


  --------------------
  -- Target signals --
  --------------------

  -- Clocks
  signal clks_non_mgt : clks_non_mgt_t;
  signal clks_mgt     : clks_mgt_t;

  -- Configuration/online signals
  signal target_configured_n : std_logic;
  signal target_gtp_online_n : std_logic;
  signal bridge_gtp_online_n : std_logic;
  signal linti_n,linto_n     : std_logic;
  signal gtp_online          : std_logic;
  signal gtp_online_cnt      : std_logic_vector(7 downto 0);
  signal mptl_online_long    : std_logic;
  signal mptl_online_long_d1 : std_logic;

  -- MPTL signals
  signal mptl_b2t : mptl_pins_t;
  signal mptl_t2b : mptl_pins_t;

  -- OCP signals
  signal direct_slave_m2s   : adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;
  signal direct_slave_s2m   : adb3_ocp_s2mT;
  signal dma_channels_m2s   : adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1) := (others => ADB3_OCP_DEFAULT_M2S);
  signal dma_channels_s2m   : adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
  signal direct_masters_m2s : adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
  signal direct_masters_s2m : adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1) := (others => ADB3_OCP_DEFAULT_S2M);

  -- Front XRM IO
  signal xrm_gpio   : xrm_gpio_t;

  -- Rear PN4 IO
  signal pn4_gpio   : pn4_gpio_t;

  -- Rear PN6 IO
  signal pn6_gpio   : pn6_gpio_t;


  component test_uber_ds
  port(
    -- OCP Interface
    ocp_clk          : in    std_logic;
    direct_slave_m2s : out   adb3_ocp_m2sT;
    direct_slave_s2m : in    adb3_ocp_s2mT;
    -- Test interface
    mptl_online_long : in    std_logic;
    ds_comp          : out   ds_comp_t;
    ds_pass          : out   ds_pass_t);
  end component;

  component test_uber_dma
  port(
    -- OCP Interface
    ocp_clk          : in    std_logic;
    dma_channels_m2s : out   adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    -- Test interface
    mptl_online_long : in    std_logic;
    dma_comp         : out   dma_comp_t;
    dma_pass         : out   dma_pass_t);
  end component;

  component test_uber_dm
  port(
    -- OCP Interface
    ocp_clk            : in    std_logic;
    direct_masters_m2s : in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
    direct_masters_s2m : out   adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1));
  end component;

  component test_uber_probes
  port(
    -- OCP Interface
    ocp_clk            : in    std_logic;
    direct_slave_m2s   : in    adb3_ocp_m2sT;
    direct_slave_s2m   : in    adb3_ocp_s2mT;
    dma_channels_m2s   : in    adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m   : in    adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1);
    direct_masters_m2s : in    adb3_ocp_m2sT_array(0 to MAX_DM_CHANNELS-1);
    direct_masters_s2m : in    adb3_ocp_s2mT_array(0 to MAX_DM_CHANNELS-1);
    -- Probe Interface
    probe_ds_err       : out   std_logic;
    probe_dma_err      : out   std_logic;
    probe_dm_err       : out   std_logic);
  end component;

  component uber
  port(
    -- Non-MGT Clock Interface
    clks_non_mgt             : in    clks_non_mgt_t;
    -- MGT Clock Interface
    clks_mgt                 : in    clks_mgt_t;
    -- MPTL Interface
    mptl_b2t                 : in    mptl_pins_t;
    mptl_t2b                 : out   mptl_pins_t;
    -- MPTL Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    mptl_dma_abort_l         : out   mptl_dma_abort_t;
    -- Interrupt Interface
    finto_n                  : in    std_logic;
    finti_n                  : out   std_logic;
    -- XRM Front Interface
    xrm_gpio                 : inout xrm_gpio_t;
    -- PN4 Rear Interface
    pn4_gpio                 : inout pn4_gpio_t;
    -- PN6 Rear Interface
    pn6_gpio                 : inout pn6_gpio_t);
  end component;

begin

  -- Generate unique reference clock frequencies
  -- 250 MHz clock
  clk_250.p <= not clk_250.p after CLK_250_PERIOD / 2;
  clk_250.n <= not clk_250.n after CLK_250_PERIOD / 2;

  -- 200 MHz clock
  clk_200.p <= not clk_200.p after CLK_200_PERIOD / 2;
  clk_200.n <= not clk_200.n after CLK_200_PERIOD / 2;

  -- 125 MHz clock
  clk_125.p <= not clk_125.p after CLK_125_PERIOD / 2;
  clk_125.n <= not clk_125.n after CLK_125_PERIOD / 2;

  -- 100 MHz clock
  clk_100.p <= not clk_100.p after CLK_100_PERIOD / 2;
  clk_100.n <= not clk_100.n after CLK_100_PERIOD / 2;

  --  80 MHz clock
  clk_80.p  <= not clk_80.p  after CLK_80_PERIOD / 2;
  clk_80.n  <= not clk_80.n  after CLK_80_PERIOD / 2;

  -- XRM clock
  clk_xrm.p  <= not clk_xrm.p after CLK_XRM_PERIOD / 2;
  clk_xrm.n  <= not clk_xrm.n after CLK_XRM_PERIOD / 2;

  -- Report design constants
  report_setup_p : process
  begin
    report "Board Type : " & board_type_t'image(BOARD_TYPE);  -- adb3_target_inc_pkg
    report "Target Use : " & target_use_t'image(TARGET_USE);  -- adb3_target_inc_pkg
    report "Master Use : " & master_use_t'image(MASTER_USE);  -- uber_tb_pkg
    report "Waiting for MPTL online....";
    wait;
  end process;


  -----------------------------------
  -- Bridge Direct Slave interface --
  -----------------------------------
  test_uber_ds_i : test_uber_ds
  port map(
    -- OCP Interface
    ocp_clk          => ocp_clk,           -- in    std_logic
    direct_slave_m2s => direct_slave_m2s,  -- out   adb3_ocp_m2sT
    direct_slave_s2m => direct_slave_s2m,  -- in    adb3_ocp_s2mT
    -- Test interface
    mptl_online_long => mptl_online_long,  -- in    std_logic
    ds_comp          => ds_comp,           -- out   ds_comp_r
    ds_pass          => ds_pass);          -- out   ds_pass_r)

  --------------------------
  -- Bridge DMA interface --
  --------------------------
  test_uber_dma_i : test_uber_dma
  port map(
    -- OCP Interface
    ocp_clk          => ocp_clk,           -- in    std_logic
    dma_channels_m2s => dma_channels_m2s,  -- out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1)
    dma_channels_s2m => dma_channels_s2m,  -- in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1)
    -- Test interface
    mptl_online_long => mptl_online_long,  -- in    std_logic
    dma_comp         => dma_comp,          -- out   dma_comp_t
    dma_pass         => dma_pass);         -- out   dma_pass_t)

  ------------------------------------
  -- Bridge Direct master interface --
  ------------------------------------

  dm32_g : if (MASTER_USE = MASTER_ON) generate
    test_uber_dm_i : test_uber_dm
    port map(
      -- OCP Interface
      ocp_clk            => ocp_clk,             -- in    std_logic
      direct_masters_m2s => direct_masters_m2s,  -- in    adb3_ocp_m2sT_array(0 to MASTER_CHANNELS-1)
      direct_masters_s2m => direct_masters_s2m); -- out   adb3_ocp_s2mT_array(0 to MASTER_CHANNELS-1))
  end generate;


  -- Monitor interrupt input and detect falling edges
  linti_n_monitor_p : process
  begin
    loop
      -- Wait for a negative edge on 'linti_n'
      wait until falling_edge(linti_n);
      report "Interrupt Monitor: Detected falling edge on linti_n";
      -- Shared variable defined in uber_tb_pkg
      linti_n_edge := '1';
    end loop;
  end process;


  -------------------------
  -- Output test results --
  -------------------------

  gen_results_p : process(ds_pass,ds_comp,dma_pass,dma_comp)
  begin
    test_passed <= ds_pass.simple_passed and
                   ds_pass.master_passed and  -- master_passed is true if not run
                   ds_pass.clock_passed and
                   ds_pass.frontio_passed and ds_pass.reario_passed and
                   ds_pass.int_passed and
                   ds_pass.info_passed and
                   ds_pass.bram_passed and
                   dma_pass.dma_write_passed and dma_pass.dma_read_passed and
                   (b_gpio_t2b = MPTL_GPIO);

    test_complete <= ds_comp.simple_complete and
                     (ds_comp.master_complete or (MASTER_USE = MASTER_OFF)) and
                     ds_comp.clock_complete and
                     ds_comp.frontio_complete and ds_comp.reario_complete and
                     ds_comp.int_complete and
                     ds_comp.info_complete and
                     ds_comp.bram_complete and
                     dma_comp.dma_write_complete and dma_comp.dma_read_complete;
  end process;


  test_results_p : process(test_complete,test_passed,probe_ds_err,probe_dm_err,probe_dma_err)
  begin
    if (test_complete) then
      assert not(probe_ds_err  = '1') report "Probe OCP Direct Slave status: ERROR."  severity error;
      assert not(probe_dm_err  = '1') report "Probe OCP Direct Master status: ERROR." severity error;
      assert not(probe_dma_err = '1') report "Probe OCP DMA status: ERROR."           severity error;

      if (test_passed) then
        assert false report "Test of design UBER completed: PASSED." severity failure;
      else
        assert false report "Test of design UBER completed: FAILED." severity failure;
      end if;
    end if;
  end process;


  ------------------------------------
  -- Target Component Instantiation --
  ------------------------------------

  non_mgt_clks_6tl_g : if (BOARD_TYPE = ADM_XRC_6TL) generate
    -- Connect test clocks to non-MGT clock inputs
    clks_non_mgt.ref_clk   <= clk_200;     -- Osc. Reference clock 200 MHz
    clks_non_mgt.lclk_s    <= clk_80.p;    -- Bridge programmable clock 40-140 MHz
    clks_non_mgt.xrm_clkin <= clk_xrm;     -- XRM reference clock
  end generate;

  non_mgt_clks_6t1_g : if (BOARD_TYPE = ADM_XRC_6T1) generate
    -- Connect test clocks to non-MGT clock inputs
    clks_non_mgt.ref_clk   <= clk_200;     -- Osc. Reference clock 200 MHz
    clks_non_mgt.xrm_clkin <= clk_xrm;     -- XRM reference clock
  end generate;

  mgt_clks_6tl_g : if (BOARD_TYPE = ADM_XRC_6TL) generate
    -- Connect test clocks to MGT clock inputs
    clks_mgt.mgt112_clk0   <= clk_125;     -- Osc. Reference clock 125 MHz
    clks_mgt.mgt113_clk0   <= clk_100;     -- PCIe Reference clock 100 MHz
    clks_mgt.mgt113_clk1   <= clk_250;     -- PCIe Reference clock 250 MHz
    clks_mgt.mgt114_clk0   <= clk_250;     -- PCIe Reference clock 250 MHz
    clks_mgt.mgt115_clk0   <= clk_125;     -- Osc. Reference clock 125 MHz
    clks_mgt.mgt116_clk0   <= clk_125;     -- Osc. Reference clock 125 MHz
    clks_mgt.mgt117_clk0   <= clk_xrm;     -- XRM Reference clock
  end generate;

  mgt_clks_6t1_g : if (BOARD_TYPE = ADM_XRC_6T1) generate
    -- Connect test clocks to MGT clock inputs
    clks_mgt.mgt113_clk0   <= clk_250;     -- Osc. Reference clock 250 MHz
    clks_mgt.mgt113_clk1   <= clk_250;     -- PCIe Reference clock 250 MHz
    clks_mgt.mgt114_clk0   <= clk_250;     -- PCIe Reference clock 250 MHz
    clks_mgt.mgt115_clk0   <= clk_250;     -- Osc. Reference clock 250 MHz
    clks_mgt.mgt116_clk0   <= clk_250;     -- Osc. Reference clock 250 MHz
    clks_mgt.mgt117_clk0   <= clk_xrm;     -- XRM Reference clock
  end generate;

  linto_n <= '1';  -- Unused

  -- Instantiate target UUT
  uut : uber
  port map(
    -- Clock Interface
    clks_non_mgt             => clks_non_mgt,        -- in    clks_non_mgt_t
    -- MGT Clock Interface
    clks_mgt                 => clks_mgt,            -- in    clks_mgt_t
    -- MPTL Interface
    mptl_b2t                 => mptl_b2t,            -- in    mptl_pins_t
    mptl_t2b                 => mptl_t2b,            -- out   mptl_pins_t
    -- MPTL Sideband Signals
    mptl_target_configured_n => target_configured_n, -- out   std_logic
    mptl_target_gtp_online_n => target_gtp_online_n, -- out   std_logic
    mptl_bridge_gtp_online_n => bridge_gtp_online_n, -- in    std_logic
    mptl_dma_abort_l         => open,                -- out   mptl_dma_abort_t
    -- Interrupt Interface
    finto_n                  => linto_n,             -- in    std_logic
    finti_n                  => linti_n,             -- out   std_logic
    -- XRM Front Interface
    xrm_gpio                 => xrm_gpio,            -- inout xrm_gpio_t
    -- PN4 Rear Interface
    pn4_gpio                 => pn4_gpio,            -- inout pn4_gpio_t
    -- PN6 Rear Interface
    pn6_gpio                 => pn6_gpio);           -- inout pn6_gpio_t)



  -----------------------------------------------
  -- Bridge Component Instantiations (Private) --
  -----------------------------------------------

  -- Connect inputs
  b_gpio_b2t    <= MPTL_GPIO;
  b_mptl_config <= (others => '0');


  ocp_clk_sim_ocp_g : if (TARGET_USE = SIM_OCP) generate
    -- Direct OCP-OCP simulation
    -- Bridge ocp clock must be target ocp clock
    ocp_clk <= mptl_t2b.target_ocp_clk;
  end generate;

  ocp_clk_sim_mptl_g : if (TARGET_USE = SIM_MPTL) generate
    -- OCP-MPTL-OCP simulation
    -- Bridge ocp clock can be different from target ocp clock
    ocp_clk <= clk_200.p;
  end generate;

  mptl_clk_par_g : if (BOARD_TYPE = ADM_XRC_6TL) generate
    -- Connect 250MHz PCIe P5 clk to mptl_clk (single ended)
    mptl_clk.clk <= clk_250.p;
  end generate;

  mptl_clk_ser_g : if (BOARD_TYPE = ADM_XRC_6T1) generate
    -- Connect 250 MHz Osc to mptl_clk (double ended)
    mptl_clk.p <= clk_250.p;
    mptl_clk.n <= clk_250.n;
  end generate;


  -- bridge interface
  mptl_if_bridge_wrap_i : mptl_if_bridge_wrap
  port map(
    -- OCP Interface
    ocp_clk                  => ocp_clk,             -- in    std_logic
    direct_slave_m2s         => direct_slave_m2s,    -- in    adb3_ocp_m2sT
    direct_slave_s2m         => direct_slave_s2m,    -- out   adb3_ocp_s2mT
    dma_channels_m2s         => dma_channels_m2s,    -- in    adb3_ocp_m2sT_array
    dma_channels_s2m         => dma_channels_s2m,    -- out   adb3_ocp_s2mT_array
    direct_masters_m2s       => direct_masters_m2s,  -- out   adb3_ocp_m2sT
    direct_masters_s2m       => direct_masters_s2m,  -- in    adb3_ocp_s2mT
    -- MPTL Interface
    mptl_b2t                 => mptl_b2t,            -- out   mptl_pins_t;
    mptl_t2b                 => mptl_t2b,            -- in    mptl_pins_t;
    mptl_clk                 => mptl_clk,            -- in    mptl_clk_t;
    -- MPTL Sideband Signals
    mptl_target_configured_n => target_configured_n, -- in    std_logic
    mptl_target_gtp_online_n => target_gtp_online_n, -- in    std_logic
    mptl_bridge_gtp_online_n => bridge_gtp_online_n, -- out   std_logic
    -- MPTL Sideband Interface
    gpio_b2t                 => b_gpio_b2t,          -- in    std_logic_vector(15 downto 0)
    gpio_t2b                 => b_gpio_t2b,          -- out   std_logic_vector(15 downto 0)
    mptl_status              => b_mptl_status,       -- out   std_logic_vector(63 downto 0))
    mptl_config              => b_mptl_config);      -- in    std_logic_vector(31 downto 0))


  -- Generate online signal
  gtp_online <= not(bridge_gtp_online_n or target_gtp_online_n);


  -- Count cycles of gtp_online to make sure it is stable
  online_count_p : process(ocp_clk)
  begin
    if rising_edge(ocp_clk) then
      if (gtp_online = '0') then
        gtp_online_cnt <= (others => '1');
      else
        if (or_reduce(gtp_online_cnt) = '1') then
          gtp_online_cnt <= gtp_online_cnt - 1;
        end if;
      end if;
      mptl_online_long <= not(or_reduce(gtp_online_cnt));
    end if;
  end process;

  -- Monitor MPTL link and stop when failure
  monitor_status_p : process(ocp_clk)
  begin
    if rising_edge(ocp_clk) then
      mptl_online_long_d1 <= mptl_online_long;
    end if;

    -- Detect falling edge on gtp_online
    assert not((mptl_online_long = '0')and(mptl_online_long_d1 = '1')) report "Test of design UBER: MPTL link failure (Undefined data)" severity failure;
  end process;


  ------------------------
  -- Bridge OCP probes --
  ------------------------

  test_uber_probes_i : test_uber_probes
  port map(
     -- OCP Interface
    ocp_clk            => ocp_clk,             -- in    std_logic
    direct_slave_m2s   => direct_slave_m2s,    -- in    adb3_ocp_m2sT
    direct_slave_s2m   => direct_slave_s2m,    -- in    adb3_ocp_s2mT
    dma_channels_m2s   => dma_channels_m2s,    -- in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1)
    dma_channels_s2m   => dma_channels_s2m,    -- in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1)
    direct_masters_m2s => direct_masters_m2s,  -- in    adb3_ocp_m2sT_array(0 to MASTER_CHANNELS-1)
    direct_masters_s2m => direct_masters_s2m,  -- in    adb3_ocp_s2mT_array(0 to MASTER_CHANNELS-1)
    -- Probe Interface
    probe_ds_err       => probe_ds_err,        -- out   std_logic;
    probe_dma_err      => probe_dma_err,       -- out   std_logic;
    probe_dm_err       => probe_dm_err);       -- out   std_logic)

end test;