--==============================================================================
--
--     File Name: uber.vhd
--       Version: 0.1
--          Date: 26 Nov 2009
--         Model: syn (Synthesisable)
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: ADB3 Target Design
--
--   Description: Illustrates:
--                  OCP Direct Slave Block:
--                    OCP Cross Clock Domain clock domain crossing
--                    OCP Reg Split address demuxing into functional blocks
--                    OCP Simple Bus If register access
--                  OCP DMA Block:
--                    OCP Mux multiplexing of multiple OCP channels
--                    OCP Simple Bus If access to BRAM
--
--   Limitations: ADM-XRC-6TL
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 26 Nov 09: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.mptl.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;
use work.uber_pkg.all;

entity uber is
port(
  -- Non-MGT Clock Interface
  clks_non_mgt             : in    clks_non_mgt_t;
  -- MGT Clock Interface
  clks_mgt                 : in    clks_mgt_t;
  -- MPTL Interface
  mptl_b2t                 : in    mptl_pins_t;
  mptl_t2b                 : out   mptl_pins_t;
  -- MPTL Sideband Signals
  mptl_target_configured_n : out   std_logic;
  mptl_target_gtp_online_n : out   std_logic;
  mptl_bridge_gtp_online_n : in    std_logic;
  mptl_dma_abort_l         : out   mptl_dma_abort_t;
  -- Interrupt Interface
  finto_n                  : in    std_logic;
  finti_n                  : out   std_logic;
  -- XRM Front Interface
  xrm_gpio                 : inout xrm_gpio_t;
  -- PN4 Rear Interface
  pn4_gpio                 : inout pn4_gpio_t;
  -- PN6 Rear Interface
  pn6_gpio                 : inout pn6_gpio_t);
end uber;


architecture synth of uber is

  -------------------
  -- Reset signals --
  -------------------
  signal pll_ref_rst : std_logic;
  signal pll_usr_rst : std_logic;

  -------------------
  -- Clock signals --
  -------------------
  signal pll_ref_clk : std_logic;
  signal pll_usr_clk : std_logic;
  signal clk_vec     : clk_vec_t;
  signal mptl_clk    : mptl_clk_t;

  ----------------------------
  -- MPTL interface signals --
  ----------------------------
  signal direct_slave_m2s     : adb3_ocp_m2sT;
  signal direct_slave_s2m     : adb3_ocp_s2mT := ADB3_OCP_DEFAULT_S2M;
  signal dma_channels_m2s     : adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
  signal dma_channels_s2m     : adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1) := (others => ADB3_OCP_DEFAULT_S2M);
  signal mptl_gpio_b2t        : std_logic_vector(15 downto 0);
  signal mptl_gpio_t2b        : std_logic_vector(15 downto 0);
  signal mptl_target_config_n : std_logic;

  --------------------------------
  -- Register interface signals --
  --------------------------------

  -- Interrupt interface
  signal interrupt     : std_logic;

  -- BRAM Direct Slave OCP interface
  signal ds_bram_s2m   : adb3_ocp_s2mT;
  signal ds_bram_m2s   : adb3_ocp_m2sT;

begin

  ---------------------
  -- Clock interface --
  ---------------------
  blk_clocks_i : blk_clocks
  port map(
    -- Clock Inputs
    clks_non_mgt => clks_non_mgt, -- in    clks_non_mgt_t
    clks_mgt     => clks_mgt,     -- in    clks_mgt_t
    -- Reset outputs
    pll_ref_rst  => pll_ref_rst,  -- out   std_logic
    pll_usr_rst  => pll_usr_rst,  -- out   std_logic
    -- Clock Outputs
    pll_ref_clk  => pll_ref_clk,  -- out   std_logic
    pll_usr_clk  => pll_usr_clk,  -- out   std_logic
    clk_vec      => clk_vec,      -- out   clk_vec_t
    mptl_clk     => mptl_clk);    -- out   mptl_clk_t


  ---------------------
  -- MPTL interface --
  ---------------------

  -- gpio loopback
  mptl_gpio_t2b <= mptl_gpio_b2t;

  -- Wrapper for mptl target interfaces
  mptl_if_target_wrap_i : mptl_if_target_wrap
  port map(
    -- OCP Interface
    ocp_clk                  => pll_ref_clk,              -- in    std_logic
    direct_slave_m2s         => direct_slave_m2s,         -- out   adb3_ocp_m2sT
    direct_slave_s2m         => direct_slave_s2m,         -- in    adb3_ocp_s2mT
    dma_channels_m2s         => dma_channels_m2s,         -- out   adb3_ocp_m2sT
    dma_channels_s2m         => dma_channels_s2m,         -- in    adb3_ocp_s2mT
    -- MPTL Interface
    mptl_t2b                 => mptl_t2b,                 -- out   mptl_pins_t;
    mptl_b2t                 => mptl_b2t,                 -- in    mptl_pins_t;
    mptl_clk                 => mptl_clk,                 -- in    mptl_clk_t;
    -- MPTL Sideband Signals
    mptl_target_configured_n => mptl_target_config_n,     -- out   std_logic
    mptl_target_gtp_online_n => mptl_target_gtp_online_n, -- out   std_logic
    mptl_bridge_gtp_online_n => mptl_bridge_gtp_online_n, -- in    std_logic
    -- MPTL Sideband Interface
    gpio_t2b                 => mptl_gpio_t2b,            -- in    std_logic_vector(15 downto 0)
    gpio_b2t                 => mptl_gpio_b2t);           -- out   std_logic_vector(15 downto 0)

  -- Hold off MPTL target configured until clocks are stable
  mptl_target_configured_n <= mptl_target_config_n or pll_ref_rst or pll_usr_rst;


  ------------------------------------
  -- Top level io signal connection --
  ------------------------------------

  mptl_dma_abort_l <= (others => '1');

  finti_n <= not(interrupt);


  ------------------------------
  -- OCP Direct Slave Channel --
  ------------------------------

  blk_direct_slave_i : blk_direct_slave
  port map(
    -- Clock interface
    pll_ref_rst         => pll_ref_rst,      -- in    std_logic
    pll_ref_clk         => pll_ref_clk,      -- in    std_logic
    pll_usr_rst         => pll_usr_rst,      -- in    std_logic
    pll_usr_clk         => pll_usr_clk,      -- in    std_logic
    -- Direct Slave Upstream OCP Interface
    direct_slave_m2s    => direct_slave_m2s, -- in    adb3_ocp_m2sT
    direct_slave_s2m    => direct_slave_s2m, -- out   adb3_ocp_s2mT
    -- Clock frequency measurement register interface
    clk_vec             => clk_vec,          -- in    clk_vec_t
    -- IO signal test register interface
    xrm_gpio            => xrm_gpio,         -- inout xrm_gpio_t
    pn4_gpio            => pn4_gpio,         -- inout pn4_gpio_t
    pn6_gpio            => pn6_gpio,         -- inout pn6_gpio_t
    -- Interrupt test register interface
    interrupt           => interrupt,        -- out   std_logic
    -- BRAM Downstream OCP interface
    ds_bram_m2s         => ds_bram_m2s,      -- out   adb3_ocp_m2sT
    ds_bram_s2m         => ds_bram_s2m);     -- in    adb3_ocp_s2mT)


  ----------------------
  -- OCP DMA Channels --
  ----------------------
  blk_dma_i : blk_dma
  port map(
    ocp_rst          => pll_ref_rst,       -- in    std_logic
    ocp_clk          => pll_ref_clk,       -- in    std_logic
    -- DMA OCP Interface
    dma_channels_m2s => dma_channels_m2s,  -- in    adb3_ocp_m2sT_array(0 to MAX_DMA_CHANNELS-1);
    dma_channels_s2m => dma_channels_s2m,  -- out   adb3_ocp_s2mT_array(0 to MAX_DMA_CHANNELS-1));
    -- BRAM Direct Slave OCP Interface
    ds_bram_m2s      => ds_bram_m2s,       -- in    adb3_ocp_m2sT
    ds_bram_s2m      => ds_bram_s2m);      -- out   adb3_ocp_s2mT);

end;